@@ -1,32 +1,23 @@
 #!/usr/bin/env python3
 
-import sys
+from pathlib import Path
+from sys import exit, argv
+from array import array
+from signal import SIGINT
+from struct import unpack
+from threading import Thread
+
+import gi
+gi.require_version('GLib', '2.0')
+from gi.repository import GLib
+
+gi.require_version('Hinawa', '2.0')
+from gi.repository import Hinawa
 
 # Qt5 python binding
 from PyQt5.QtWidgets import QApplication, QWidget, QHBoxLayout, QVBoxLayout
 from PyQt5.QtWidgets import QToolButton, QGroupBox, QLineEdit, QLabel
 
-# Hinawa-2.0 gir
-import gi
-gi.require_version('Hinawa', '2.0')
-from gi.repository import Hinawa
-
-# to handle UNIX signal
-from gi.repository import GLib
-import signal
-
-from array import array
-
-import glob
-
-# helper function
-def get_array():
-    # The width with 'L' parameter is depending on environment.
-    arr = array('L')
-    if arr.itemsize is not 4:
-        arr = array('I')
-    return arr
-
 # query sound devices and get FireWire sound unit
 snd_specific_types = {
     Hinawa.SndUnitType.DICE:        Hinawa.SndDice,
@@ -34,72 +25,76 @@ snd_specific_types = {
     Hinawa.SndUnitType.DIGI00X:     Hinawa.SndDg00x,
     Hinawa.SndUnitType.MOTU:        Hinawa.SndMotu,
 }
-for fullpath in glob.glob('/dev/snd/hw*'):
+for p in Path('/dev/snd/').glob('hw*'):
+    fullpath = str(p)
     try:
-        snd_unit = Hinawa.SndUnit()
-        snd_unit.open(fullpath)
+        unit = Hinawa.SndUnit()
+        unit.open(fullpath)
     except:
-        del snd_unit
+        del unit
         continue
 
-    snd_type = snd_unit.get_property('type')
+    snd_type = unit.get_property('type')
     if snd_type in snd_specific_types:
-        del snd_unit
-        snd_unit = snd_specific_types[snd_type]()
+        del unit
+        unit = snd_specific_types[snd_type]()
         try:
-            snd_unit.open(fullpath)
+            unit.open(fullpath)
         except:
-            del snd_unit
+            del unit
             continue
     break
 
-if 'snd_unit' not in locals():
+if 'unit' not in locals():
     print('No sound FireWire devices found.')
-    sys.exit()
+    exit()
 
-# create sound unit
-def handle_lock_status(snd_unit, status):
+# Start a thread to dispatch events from the sound unit and node.
+ctx = GLib.MainContext.new()
+unit_src = unit.create_source()
+unit_src.attach(ctx)
+node = unit.get_node()
+node_src = node.create_source()
+node_src.attach(ctx)
+dispatcher = GLib.MainLoop.new(ctx, False)
+dispatch_th = Thread(target=lambda d: d.run(), args=(dispatcher,))
+dispatch_th.start()
+
+# Setup sound unit.
+def handle_lock_status(unit, status):
     if status:
         print("streaming is locked.");
     else:
         print("streaming is unlocked.");
-def handle_disconnected(snd_unit):
+def handle_disconnected(unit):
     print('disconnected')
-    app.quit()
+unit.connect("lock-status", handle_lock_status)
+unit.connect("disconnected", handle_disconnected)
+
 print('Sound device info:')
-print(' type:\t\t{0}'.format(snd_unit.get_property("type").value_nick))
-print(' card:\t\t{0}'.format(snd_unit.get_property("card")))
-print(' device:\t{0}'.format(snd_unit.get_property("device")))
-print(' GUID:\t\t{0:016x}'.format(snd_unit.get_property("guid")))
-snd_unit.connect("lock-status", handle_lock_status)
-snd_unit.connect("disconnected", handle_disconnected)
-print('\nIEEE1394 Unit info:')
+print(' type:\t\t{0}'.format(unit.get_property("type").value_nick))
+print(' card:\t\t{0}'.format(unit.get_property("card")))
+print(' device:\t{0}'.format(unit.get_property("device")))
+print(' GUID:\t\t{0:016x}'.format(unit.get_property("guid")))
+
+# Setup firewire node.
+def handle_bus_update(node):
+    print('bus-reset: generation {0}'.format(node.get_property('generation')))
+node.connect("bus-update", handle_bus_update)
+
+print('\nIEEE1394 node info:')
 print(' Node IDs:')
-print('  self:\t\t{0:04x}'.format(snd_unit.get_property('node-id')))
-print('  local:\t{0:04x}'.format(snd_unit.get_property('local-node-id')))
-print('  root:\t\t{0:04x}'.format(snd_unit.get_property('root-node-id')))
-print('  bus-manager:\t{0:04x}'.format(snd_unit.get_property('bus-manager-node-id')))
-print('  ir-manager:\t{0:04x}'.format(snd_unit.get_property('ir-manager-node-id')))
-print('  generation:\t{0}'.format(snd_unit.get_property('generation')))
+print('  self:\t\t{:04x}'.format(node.get_property('node-id')))
+print('  local:\t{:04x}'.format(node.get_property('local-node-id')))
+print('  root:\t\t{:04x}'.format(node.get_property('root-node-id')))
+print('  bus-manager:\t{:04x}'.format(node.get_property('bus-manager-node-id')))
+print('  ir-manager:\t{:04x}'.format(node.get_property('ir-manager-node-id')))
+print('  generation:\t{}'.format(node.get_property('generation')))
 print(' Config ROM:')
-config_rom = snd_unit.get_config_rom()
-for i in range(len(config_rom) // 4):
-    print('  [{0:016x}]: {1:02x}{2:02x}{3:02x}{4:02x}'.format(
-        0xfffff0000000 + i * 4, config_rom[i * 4], config_rom[i * 4 + 1],
-        config_rom[i * 4 + 2], config_rom[i * 4 + 3]))
-
-# create FireWire unit
-def handle_bus_update(snd_unit):
-    print('bus-reset: generation {0}'.format(snd_unit.get_property('generation')))
-snd_unit.connect("bus-update", handle_bus_update)
-
-# start listening
-try:
-    snd_unit.listen()
-except Exception as e:
-    print(e)
-    sys.exit()
-print(" listening:\t{0}".format(snd_unit.get_property('listening')))
+config_rom = node.get_config_rom()
+quads = unpack('>{}I'.format(len(config_rom) // 4), config_rom)
+for i, q in enumerate(quads):
+    print('  0xfffff000{:04x}: {:08x}'.format(i * 4, q))
 
 # create firewire responder
 resp = Hinawa.FwResp()
@@ -110,11 +105,11 @@ def handle_request(resp, tcode):
         print(' [{0:02d}]: 0x{1:02x}'.format(i, req_frame[i]))
     return Hinawa.FwRcode.COMPLETE
 try:
-    resp.register(snd_unit, 0xfffff0000d00, 0x100)
+    resp.reserve(node, 0xfffff0000d00, 0x100)
     resp.connect('requested', handle_request)
 except Exception as e:
     print(e)
-    sys.exit()
+    exit()
 
 # create firewire requester
 req = Hinawa.FwReq()
@@ -125,32 +120,38 @@ fcp_types = (
     Hinawa.SndUnitType.BEBOB,
     Hinawa.SndUnitType.OXFW,
 )
-if snd_unit.get_property('type') in fcp_types:
+if unit.get_property('type') in fcp_types:
     fcp = Hinawa.FwFcp()
     try:
-        fcp.listen(snd_unit)
+        fcp.bind(node)
     except Exception as e:
         print(e)
-        sys.exit()
+        exit()
     request = bytes([0x01, 0xff, 0x19, 0x00, 0xff, 0xff, 0xff, 0xff])
+    response = bytearray(8)
     try:
-        response = fcp.transact(request)
+        response = fcp.transaction(request, response)
     except Exception as e:
         print(e)
-        sys.exit()
+        exit()
     print('FCP Response:')
-    for i in range(len(response)):
-        print(' [{0:02d}]: 0x{1:02x}'.format(i, response[i]))
+    for i, b in enumerate(response):
+        print(' [{0:02d}]: 0x{1:02x}'.format(i, b))
+    fcp.unbind()
+    del fcp
 
 # Echo Fireworks Transaction
-if snd_unit.get_property("type") is 2:
-    args = get_array()
+if unit.get_property('type') is Hinawa.SndUnitType.FIREWORKS:
+    args = array('I')
     args.append(5)
+    params = array('I')
+    params.append(0xffffffff)
+    params.append(0xffffffff)
     try:
-        params = snd_unit.transact(6, 1, args)
+        params = unit.transaction(6, 1, args, params)
     except Exception as e:
         print(e)
-        sys.exit()
+        exit()
     print('Echo Fireworks Transaction Response:')
     for i in range(len(params)):
         print(" [{0:02d}]: {1:08x}".format(i, params[i]))
@@ -158,29 +159,29 @@ if snd_unit.get_property("type") is 2:
 # Dice notification
 def handle_notification(self, message):
     print("Dice Notification: {0:08x}".format(message))
-if snd_unit.get_property('type') is Hinawa.SndUnitType.DICE:
-    snd_unit.connect('notified', handle_notification)
-    args = get_array()
+if unit.get_property('type') is Hinawa.SndUnitType.DICE:
+    unit.connect('notified', handle_notification)
+    args = array('I')
     args.append(0x0000030c)
     try:
         # The address of clock in Impact Twin
-        snd_unit.transact(0xffffe0000074, args, 0x00000020)
+        unit.transaction(0xffffe0000074, args, 0x00000020)
     except Exception as e:
         print(e)
-        sys.exit()
+        exit()
 
 # Dg00x message
 def handle_message(self, message):
     print("Dg00x Messaging {0:08x}".format(message));
-if snd_unit.get_property('type') is Hinawa.SndUnitType.DIGI00X:
+if unit.get_property('type') is Hinawa.SndUnitType.DIGI00X:
     print('hear message')
-    snd_unit.connect('message', handle_message)
+    unit.connect('message', handle_message)
 
 # Motu notification and status
 def handle_motu_notification(self, message):
     print("Motu Notification: {0:08x}".format(message))
-if snd_unit.get_property('type') is Hinawa.SndUnitType.MOTU:
-    snd_unit.connect('notified', handle_motu_notification)
+if unit.get_property('type') is Hinawa.SndUnitType.MOTU:
+    unit.connect('notified', handle_motu_notification)
 
 # GUI
 class Sample(QWidget):
@@ -221,7 +222,7 @@ class Sample(QWidget):
         buttom_layout.addWidget(self.value)
 
         # handle unix signal
-        GLib.unix_signal_add(GLib.PRIORITY_HIGH, signal.SIGINT, \
+        GLib.unix_signal_add(GLib.PRIORITY_HIGH, SIGINT,
                              self.handle_unix_signal, None)
 
     def handle_unix_signal(self, user_data):
@@ -230,16 +231,18 @@ class Sample(QWidget):
     def transact(self, val):
         try:
             addr = int(self.addr.text(), 16)
-            vals = req.read(snd_unit, addr, 4)
+            frames = bytearray(4);
+            frames = req.transaction(node, Hinawa.FwTcode.READ_QUADLET_REQUEST,
+                                     addr, 4, frames)
         except Exception as e:
             print(e)
             return
 
-        label = '0x{0:02x}{1:02x}{2:02x}{3:02x}'.format(vals[0], vals[1], vals[2], vals[3])
+        label = '0x{:08x}'.format(unpack('>I', frames)[0])
         self.value.setText(label)
         print(self.value.text())
 
-app = QApplication(sys.argv)
+app = QApplication(argv)
 sample = Sample()
 
 sample.show()
@@ -249,15 +252,20 @@ del app
 del sample
 print('delete application object')
 
-snd_unit.unlisten()
-del snd_unit
-print('delete snd_unit object')
+dispatcher.quit()
+dispatch_th.join()
+del unit_src
+del node_src
+del dispatcher
 
-resp.unregister()
+resp.release()
 del resp
 print('delete fw_resp object')
 
 del req
 print('delete fw_req object')
 
-sys.exit()
+del node
+del unit
+
+exit()
