@@ -52,14 +52,16 @@ enum avc_status {
 };
 
 struct fcp_transaction {
-	GByteArray *req_frame;	/* Request frame */
-	GByteArray *resp_frame;	/* Response frame */
+	const guint8 *req_frame;
+	guint req_frame_size;
+	guint8 *resp_frame;
+	guint resp_frame_size;
 	GCond cond;
 	GMutex mutex;
 };
 
 struct _HinawaFwFcpPrivate {
-	HinawaFwUnit *unit;
+	HinawaFwNode *node;
 
 	GList *transactions;
 	GMutex transactions_mutex;
@@ -110,7 +112,7 @@ static void fw_fcp_finalize(GObject *obj)
 {
 	HinawaFwFcp *self = HINAWA_FW_FCP(obj);
 
-	hinawa_fw_fcp_unlisten(self);
+	hinawa_fw_fcp_unbind(self);
 
 	G_OBJECT_CLASS(hinawa_fw_fcp_parent_class)->finalize(obj);
 }
@@ -159,19 +161,33 @@ HinawaFwFcp *hinawa_fw_fcp_new(void)
 	return g_object_new(HINAWA_TYPE_FW_FCP, NULL);
 }
 
+#define NOTIFY_ENOBUFS	1
+
 /**
- * hinawa_fw_fcp_transact:
- * @self: A #HinawaFwFcp
- * @req_frame:  (element-type guint8) (array) (in): a byte frame for request
- * @resp_frame: (element-type guint8) (array) (out caller-allocates): a byte
- *		frame for response
- * @exception: A #GError
+ * hinawa_fw_fcp_transaction:
+ * @self: A #HinawaFwFcp.
+ * @req_frame: (array length=req_frame_size)(in): An array with elements for
+ *	       request byte data. The value of this argument should point to
+ *	       the array and immutable.
+ * @req_frame_size: The size of array for request in byte unit.
+ * @resp_frame: (array length=resp_frame_size)(inout): An array with elements
+ *		for response byte data. Callers should give it for buffer with
+ *		enough space against the request since this library performs no
+ *		reallocation. Due to the reason, the value of this argument
+ *		should point to the pointer to the array and immutable. The
+ *		content of array is mutable.
+ * @resp_frame_size: The size of array for response in byte unit. The value of
+ *		     this argument should point to the numerical number and
+ *		     mutable.
+ * @exception: A #GError.
  *
- * Execute fCP transaction.
+ * Execute FCP transaction.
+ * Since: 1.4.
  */
-void hinawa_fw_fcp_transact(HinawaFwFcp *self,
-			    GByteArray *req_frame, GByteArray *resp_frame,
-			    GError **exception)
+void hinawa_fw_fcp_transaction(HinawaFwFcp *self,
+			       const guint8 *req_frame, guint req_frame_size,
+			       guint8 *const *resp_frame, guint *resp_frame_size,
+			       GError **exception)
 {
 	HinawaFwFcpPrivate *priv;
 	HinawaFwReq *req;
@@ -182,22 +198,31 @@ void hinawa_fw_fcp_transact(HinawaFwFcp *self,
 	g_return_if_fail(HINAWA_IS_FW_FCP(self));
 	priv = hinawa_fw_fcp_get_instance_private(self);
 
-	if (!req_frame || !resp_frame ||
-	    req_frame->len > FCP_MAXIMUM_FRAME_BYTES) {
+	if (req_frame == NULL || *resp_frame == NULL ||
+	    req_frame_size == 0 || *resp_frame_size == 0 ||
+	    req_frame_size > FCP_MAXIMUM_FRAME_BYTES) {
 		raise(exception, EINVAL);
 		return;
 	}
 
-	req = g_object_new(HINAWA_TYPE_FW_REQ, NULL);
-
-	/* Prepare for an entry of FCP transaction. */
-	trans.req_frame = req_frame;
-	trans.resp_frame = resp_frame;
-
 	g_object_get(G_OBJECT(self), "timeout", &timeout_ms, NULL);
 
+	// NOTE: This is too rough estimation that 66 % of the given timeout
+	// can be consumed by write/response subaction for request transaction
+	// of FCP command and the rest by write subaction for request
+	// transaction of FCP response. This is just a heuristic because
+	// actual transactions are internally handled by abstraction layer of
+	// Linux FireWire subsystem.
+	req = g_object_new(HINAWA_TYPE_FW_REQ, "timeout", timeout_ms * 2 / 3, NULL);
+
+	// Prepare for an entry of FCP transaction.
+	trans.req_frame = req_frame;
+	trans.req_frame_size = req_frame_size;
+	trans.resp_frame = *resp_frame;
+	trans.resp_frame_size = *resp_frame_size;
+
 	// This predicates against suprious wakeup.
-	g_byte_array_remove_range(trans.resp_frame, 0, trans.resp_frame->len);
+	trans.resp_frame[0] = 0x00;
 	g_cond_init(&trans.cond);
 	g_mutex_init(&trans.mutex);
 	g_mutex_lock(&trans.mutex);
@@ -207,28 +232,37 @@ void hinawa_fw_fcp_transact(HinawaFwFcp *self,
 	priv->transactions = g_list_prepend(priv->transactions, &trans);
 	g_mutex_unlock(&priv->transactions_mutex);
 
-	/* Send this request frame. */
-	hinawa_fw_req_write(req, priv->unit, FCP_REQUEST_ADDR, trans.req_frame,
-			    exception);
+	// Send this request frame.
+	hinawa_fw_req_transaction(req, priv->node,
+			HINAWA_FW_TCODE_WRITE_BLOCK_REQUEST,
+			FCP_REQUEST_ADDR, trans.req_frame_size,
+			(guint8 *const *)&trans.req_frame, &trans.req_frame_size,
+			exception);
 	if (*exception)
 		goto end;
 deferred:
 	expiration = g_get_monotonic_time() +
 		     timeout_ms * G_TIME_SPAN_MILLISECOND;
 
-	while (trans.resp_frame->len == 0) {
+	while (trans.resp_frame[0] == 0x00) {
 		// NOTE: Timeout at bus-reset, illegally.
 		if (!g_cond_wait_until(&trans.cond, &trans.mutex, expiration))
 			break;
 	}
-	if (trans.resp_frame->len == 0) {
+	if (trans.resp_frame[0] == 0x00) {
 		raise(exception, ETIMEDOUT);
-	} else if (trans.resp_frame->data[0] == AVC_STATUS_INTERIM) {
-		/* It's a deffered transaction, wait again. */
-		g_byte_array_remove_range(trans.resp_frame, 0,
-					  trans.resp_frame->len);
+	} else if (trans.resp_frame[0] == AVC_STATUS_INTERIM) {
+		// It's a deffered transaction, wait again.
+		trans.resp_frame[0] = 0x00;
+		// Although the timeout is infinite in 1394 TA specification,
+		// use the finite value for safe.
 		goto deferred;
 	}
+
+	if (trans.resp_frame_size > NOTIFY_ENOBUFS)
+		*resp_frame_size = trans.resp_frame_size;
+	else
+		raise(exception, ENOBUFS);
 end:
 	g_mutex_unlock(&trans.mutex);
 	g_cond_clear(&trans.cond);
@@ -243,6 +277,30 @@ end:
 	g_clear_object(&req);
 }
 
+/**
+ * hinawa_fw_fcp_transact:
+ * @self: A #HinawaFwFcp
+ * @req_frame:  (element-type guint8) (array) (in): a byte frame for request
+ * @resp_frame: (element-type guint8) (array) (out caller-allocates): a byte
+ *		frame for response
+ * @exception: A #GError
+ *
+ * Execute fCP transaction.
+ * Deprecated: 1.4: Use hinawa_fw_fcp_transaction() instead,
+ */
+void hinawa_fw_fcp_transact(HinawaFwFcp *self,
+			    GByteArray *req_frame, GByteArray *resp_frame,
+			    GError **exception)
+{
+	// Some vendor specific FCP transaction has such response that the
+	// size does not equal to the size of request...
+	g_byte_array_set_size(resp_frame, FCP_MAXIMUM_FRAME_BYTES);
+
+	hinawa_fw_fcp_transaction(self, req_frame->data, req_frame->len,
+				  &(resp_frame->data), &(resp_frame->len),
+				  exception);
+}
+
 static HinawaFwRcode handle_response(HinawaFwResp *resp, HinawaFwTcode tcode)
 {
 	HinawaFwFcp *self = HINAWA_FW_FCP(resp);
@@ -262,12 +320,18 @@ static HinawaFwRcode handle_response(HinawaFwResp *resp, HinawaFwTcode tcode)
 	for (entry = priv->transactions; entry != NULL; entry = entry->next) {
 		trans = (struct fcp_transaction *)entry->data;
 
-		if (g_array_index(trans->req_frame, guint8, 1) == req_frame[1] &&
-		    g_array_index(trans->req_frame, guint8, 2) == req_frame[2]) {
+		if (trans->req_frame[1] == req_frame[1] &&
+		    trans->req_frame[2] == req_frame[2]) {
 			g_mutex_lock(&trans->mutex);
-			g_byte_array_append(trans->resp_frame,
-					    (guint8 *)req_frame, length);
+
+			// To notify ENOBUFS.
+			if (trans->resp_frame_size < length)
+				length = NOTIFY_ENOBUFS;
+
+			memcpy((void *)trans->resp_frame, req_frame, length);
+			trans->resp_frame_size = length;
 			g_cond_signal(&trans->cond);
+
 			g_mutex_unlock(&trans->mutex);
 			break;
 		}
@@ -281,34 +345,80 @@ static HinawaFwRcode handle_response(HinawaFwResp *resp, HinawaFwTcode tcode)
 }
 
 /**
- * hinawa_fw_fcp_listen:
- * @self: A #HinawaFwFcp
- * @unit: A #HinawaFwUnit
- * @exception: A #GError
+ * hinawa_fw_fcp_bind:
+ * @self: A #HinawaFwFcp.
+ * @node: A #HinawaFwNode.
+ * @exception: A #GError.
  *
  * Start to listen to FCP responses.
  */
-void hinawa_fw_fcp_listen(HinawaFwFcp *self, HinawaFwUnit *unit,
-			  GError **exception)
+void hinawa_fw_fcp_bind(HinawaFwFcp *self, HinawaFwNode *node,
+			GError **exception)
 {
 	HinawaFwFcpPrivate *priv;
 
 	g_return_if_fail(HINAWA_IS_FW_FCP(self));
 	priv = hinawa_fw_fcp_get_instance_private(self);
 
-	priv->unit = g_object_ref(unit);
-
-	hinawa_fw_resp_register(HINAWA_FW_RESP(self), priv->unit,
+	if (priv->node == NULL) {
+		hinawa_fw_resp_reserve(HINAWA_FW_RESP(self), node,
 				FCP_RESPOND_ADDR, FCP_MAXIMUM_FRAME_BYTES,
 				exception);
-	if (*exception != NULL) {
-		g_object_unref(priv->unit);
-		priv->unit = NULL;
-		return;
-	}
+		if (*exception != NULL)
+			return;
+		priv->node = g_object_ref(node);
 
-	g_mutex_init(&priv->transactions_mutex);
-	priv->transactions = NULL;
+		g_mutex_init(&priv->transactions_mutex);
+		priv->transactions = NULL;
+	}
+}
+
+/**
+ * hinawa_fw_fcp_unbind:
+ * @self: A #HinawaFwFcp.
+ *
+ * Stop to listen to FCP responses.
+ *
+ * Since: 1.4.
+ */
+void hinawa_fw_fcp_unbind(HinawaFwFcp *self)
+{
+	HinawaFwFcpPrivate *priv;
+
+	g_return_if_fail(HINAWA_IS_FW_FCP(self));
+	priv = hinawa_fw_fcp_get_instance_private(self);
+
+	if (priv->node != NULL) {
+		hinawa_fw_resp_release(HINAWA_FW_RESP(self));
+
+		g_object_unref(priv->node);
+		priv->node = NULL;
+	}
+}
+
+/**
+ * hinawa_fw_fcp_listen:
+ * @self: A #HinawaFwFcp
+ * @unit: A #HinawaFwUnit
+ * @exception: A #GError
+ *
+ * Start to listen to FCP responses.
+ *
+ * Deprecated: 1.4: Use hinawa_fw_fcp_bind() with an instance of HinawaFwNode,
+ *		    instead.
+ */
+void hinawa_fw_fcp_listen(HinawaFwFcp *self, HinawaFwUnit *unit,
+			  GError **exception)
+{
+	HinawaFwNode *node;
+
+	g_return_if_fail(HINAWA_IS_FW_FCP(self));
+
+	hinawa_fw_unit_get_node(unit, &node);
+
+	g_object_ref(node);
+	hinawa_fw_fcp_bind(self, node, exception);
+	g_object_unref(node);
 }
 
 /**
@@ -316,18 +426,10 @@ void hinawa_fw_fcp_listen(HinawaFwFcp *self, HinawaFwUnit *unit,
  * @self: A #HinawaFwFcp
  *
  * Stop to listen to FCP responses.
+ *
+ * Deprecated: 1.4: Use hinawa_fw_fcp_unbind(), instead.
  */
 void hinawa_fw_fcp_unlisten(HinawaFwFcp *self)
 {
-	HinawaFwFcpPrivate *priv;
-
-	g_return_if_fail(HINAWA_IS_FW_FCP(self));
-	priv = hinawa_fw_fcp_get_instance_private(self);
-
-	hinawa_fw_resp_unregister(HINAWA_FW_RESP(self));
-
-	if (priv->unit != NULL) {
-		g_object_unref(priv->unit);
-		priv->unit = NULL;
-	}
+	hinawa_fw_fcp_unbind(self);
 }
