@@ -8,12 +8,10 @@
 /**
  * SECTION:fw_req
  * @Title: HinawaFwReq
- * @Short_description: A transaction executor to a FireWire unit
+ * @Short_description: A transaction executor to a FireWire unit.
  *
- * A HinawaFwReq supports three types of transactions in IEEE 1212:
- *  - read
- *  - write
- *  - lock
+ * A HinawaFwReq supports some types of transactions in IEEE 1212. Mainly for
+ * read, write and lock operations.
  *
  * This class is an application of Linux FireWire subsystem. All of operations
  * utilize ioctl(2) with subsystem specific request commands.
@@ -111,7 +109,7 @@ static void hinawa_fw_req_class_init(HinawaFwReqClass *klass)
 		g_param_spec_uint("timeout", "timeout",
 				  "An elapse to expire waiting for response by ms unit.",
 				  10, UINT_MAX,
-				  10,
+				  200,
 				  G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
 	g_object_class_install_properties(gobject_class,
@@ -139,38 +137,112 @@ HinawaFwReq *hinawa_fw_req_new(void)
 	return g_object_new(HINAWA_TYPE_FW_REQ, NULL);
 }
 
-static void fw_req_transact(HinawaFwReq *self, HinawaFwUnit *unit, int tcode,
-			    guint64 addr, guint8 *frame, guint length,
-			    GError **exception)
+/**
+ * hinawa_fw_req_transaction:
+ * @self: A #HinawaFwReq.
+ * @node: A #HinawaFwNode.
+ * @tcode: A transaction code of HinawaFwTcode.
+ * @addr: A destination address of target device
+ * @length: The range of address in byte unit.
+ * @frame: (array length=frame_size)(inout): An array with elements for byte
+ *	   data. Callers should give it for buffer with enough space against the
+ *	   request since this library performs no reallocation. Due to the
+ *	   reason, the value of this argument should point to the pointer to the
+ *	   array and immutable. The content of array is mutable for read and
+ *	   lock transaction.
+ * @frame_size: The size of array in byte unit. The value of this argument
+ *		should point to the numerical number and mutable for read and
+ *		lock transaction.
+ * @exception: A #GError.
+ *
+ * Execute transactions to the given node according to given code.
+ * Since: 1.4.
+ */
+void hinawa_fw_req_transaction(HinawaFwReq *self, HinawaFwNode *node,
+			       HinawaFwTcode tcode, guint64 addr, guint length,
+			       guint8 *const *frame, guint *frame_size,
+			       GError **exception)
 {
 	struct fw_cdev_send_request req = {0};
-	HinawaFwReqPrivate *priv = hinawa_fw_req_get_instance_private(self);
+	HinawaFwReqPrivate *priv;
 	guint64 generation;
 	guint64 expiration;
 	int err = 0;
 
-	/* Setup a private structure. */
+	g_return_if_fail(HINAWA_IS_FW_REQ(self));
+	priv = hinawa_fw_req_get_instance_private(self);
+
+	if (length == 0 || *frame == NULL || *frame_size == 0) {
+		raise(exception, EINVAL);
+		return;
+	}
+
+	// Should be aligned to quadlet.
+	if (tcode == HINAWA_FW_TCODE_WRITE_QUADLET_REQUEST ||
+	    tcode == HINAWA_FW_TCODE_READ_QUADLET_REQUEST ||
+	    tcode == HINAWA_FW_TCODE_LOCK_MASK_SWAP ||
+	    tcode == HINAWA_FW_TCODE_LOCK_COMPARE_SWAP ||
+	    tcode == HINAWA_FW_TCODE_LOCK_FETCH_ADD ||
+	    tcode == HINAWA_FW_TCODE_LOCK_LITTLE_ADD ||
+	    tcode == HINAWA_FW_TCODE_LOCK_BOUNDED_ADD ||
+	    tcode == HINAWA_FW_TCODE_LOCK_WRAP_ADD ||
+	    tcode == HINAWA_FW_TCODE_LOCK_VENDOR_DEPENDENT) {
+		if ((addr & 0x3) || (length & 0x3)) {
+			raise(exception, EINVAL);
+			return;
+		}
+	}
+
+	// Should have enough space for read/written data.
+	if (tcode == HINAWA_FW_TCODE_READ_QUADLET_REQUEST ||
+	    tcode == HINAWA_FW_TCODE_READ_BLOCK_REQUEST ||
+	    tcode == HINAWA_FW_TCODE_WRITE_QUADLET_REQUEST ||
+	    tcode == HINAWA_FW_TCODE_WRITE_BLOCK_REQUEST) {
+		if (*frame_size < length) {
+			raise(exception, ENOBUFS);
+			return;
+		}
+	} else if (tcode == HINAWA_FW_TCODE_LOCK_MASK_SWAP ||
+		   tcode == HINAWA_FW_TCODE_LOCK_COMPARE_SWAP ||
+		   tcode == HINAWA_FW_TCODE_LOCK_FETCH_ADD ||
+		   tcode == HINAWA_FW_TCODE_LOCK_LITTLE_ADD ||
+		   tcode == HINAWA_FW_TCODE_LOCK_BOUNDED_ADD ||
+		   tcode == HINAWA_FW_TCODE_LOCK_WRAP_ADD ||
+		   tcode == HINAWA_FW_TCODE_LOCK_VENDOR_DEPENDENT) {
+		if (*frame_size < length * 2) {
+			raise(exception, ENOBUFS);
+			return;
+		}
+		length *= 2;
+	} else {
+		// Not supported due to no test.
+		raise(exception, ENOTSUP);
+		return;
+	}
+
+	// Setup a private structure.
 	if (tcode == TCODE_READ_QUADLET_REQUEST ||
 	    tcode == TCODE_READ_BLOCK_REQUEST) {
-		priv->frame = frame;
+		priv->frame = *frame;
 		priv->length = length;
 		req.data = (guint64)NULL;
 	} else {
 		priv->frame = NULL;
-		req.data = (guint64)frame;
+		req.data = (guint64)(*frame);
 	}
 
-	/* Get unit properties. */
-	g_object_get(G_OBJECT(unit), "generation", &generation, NULL);
+	// Get unit properties.
+	g_object_ref(node);
+	g_object_get(G_OBJECT(node), "generation", &generation, NULL);
 
-	/* Setup a transaction structure. */
+	// Setup a transaction structure.
 	req.tcode = tcode;
 	req.length = length;
 	req.offset = addr;
 	req.closure = (guint64)self;
 	req.generation = generation;
 
-	/* Timeout is set in advance as a parameter of this object. */
+	// Timeout is set in advance as a parameter of this object.
 	expiration = g_get_monotonic_time() +
 					priv->timeout * G_TIME_SPAN_MILLISECOND;
 
@@ -179,13 +251,13 @@ static void fw_req_transact(HinawaFwReq *self, HinawaFwUnit *unit, int tcode,
 	g_cond_init(&priv->cond);
 	g_mutex_lock(&priv->mutex);
 
-	/* Send this transaction. */
-	hinawa_fw_unit_ioctl(unit, FW_CDEV_IOC_SEND_REQUEST, &req, &err);
+	// Send this transaction.
+	hinawa_fw_node_ioctl(node, FW_CDEV_IOC_SEND_REQUEST, &req, &err);
 	if (err < 0) {
 		g_mutex_unlock(&priv->mutex);
 		g_cond_clear(&priv->cond);
 		raise(exception, err);
-		return;
+		goto end;
 	}
 
 	while (priv->rcode == G_MAXUINT) {
@@ -200,7 +272,7 @@ static void fw_req_transact(HinawaFwReq *self, HinawaFwUnit *unit, int tcode,
 
 	if (priv->rcode == G_MAXUINT) {
 		raise(exception, ETIMEDOUT);
-		return;
+		goto end;
 	}
 
 	if (priv->rcode != RCODE_COMPLETE) {
@@ -211,6 +283,12 @@ static void fw_req_transact(HinawaFwReq *self, HinawaFwUnit *unit, int tcode,
 				"%d: %s", __LINE__, rcode_labels[priv->rcode]);
 		}
 	}
+
+	if (tcode == HINAWA_FW_TCODE_WRITE_QUADLET_REQUEST &&
+	    tcode == HINAWA_FW_TCODE_WRITE_BLOCK_REQUEST)
+		*frame_size = length;
+end:
+	g_object_unref(node);
 }
 
 /**
@@ -222,28 +300,24 @@ static void fw_req_transact(HinawaFwReq *self, HinawaFwUnit *unit, int tcode,
  * @exception: A #GError
  *
  * Execute write transactions to the given unit.
+ * Deprecated: 1.4: Use hinawa_fw_req_transaction() instead.
  */
 void hinawa_fw_req_write(HinawaFwReq *self, HinawaFwUnit *unit, guint64 addr,
 			 GByteArray *frame, GError **exception)
 {
+	HinawaFwNode *node;
 	int tcode;
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
-
-	if (!frame) {
-		raise(exception, EINVAL);
-		return;
-	}
-
 	if (frame->len == 4 && !(addr & 0x03))
 		tcode = TCODE_WRITE_QUADLET_REQUEST;
 	else
 		tcode = TCODE_WRITE_BLOCK_REQUEST;
 
-	g_object_ref(unit);
-	fw_req_transact(self, unit, tcode, addr, frame->data, frame->len,
-			exception);
-	g_object_unref(unit);
+	hinawa_fw_unit_get_node(unit, &node);
+	g_object_ref(node);
+	hinawa_fw_req_transaction(self, node, tcode, addr, frame->len,
+				  &(frame->data), &(frame->len), exception);
+	g_object_unref(node);
 }
 
 /**
@@ -256,30 +330,26 @@ void hinawa_fw_req_write(HinawaFwReq *self, HinawaFwUnit *unit, guint64 addr,
  * @exception: A #GError
  *
  * Execute read transaction to the given unit.
+ * Deprecated: 1.4: Use hinawa_fw_req_transaction() instead.
  */
 void hinawa_fw_req_read(HinawaFwReq *self, HinawaFwUnit *unit, guint64 addr,
 			GByteArray *frame, guint length, GError **exception)
 {
+	HinawaFwNode *node;
 	int tcode;
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
-
-	if (!frame) {
-		raise(exception, EINVAL);
-		return;
-	}
-
-	g_byte_array_set_size(frame, length);
-
-	if (frame->len == 4 && !(addr & 0x03))
+	if (length == 4 && !(addr & 0x03))
 		tcode = TCODE_READ_QUADLET_REQUEST;
 	else
 		tcode = TCODE_READ_BLOCK_REQUEST;
 
-	g_object_ref(unit);
-	fw_req_transact(self, unit, tcode, addr, frame->data, frame->len,
-			exception);
-	g_object_unref(unit);
+	g_byte_array_set_size(frame, length);
+
+	hinawa_fw_unit_get_node(unit, &node);
+	g_object_ref(node);
+	hinawa_fw_req_transaction(self, node, tcode, addr, frame->len,
+				  &(frame->data), &(frame->len), exception);
+	g_object_unref(node);
 }
 
 /**
@@ -292,44 +362,20 @@ void hinawa_fw_req_read(HinawaFwReq *self, HinawaFwUnit *unit, guint64 addr,
  * @exception: A #GError
  *
  * Execute lock transaction to the given unit.
+ * Deprecated: 1.4: Use hinawa_fw_req_transaction() instead.
  */
 void hinawa_fw_req_lock(HinawaFwReq *self, HinawaFwUnit *unit,
 			guint64 addr, GByteArray **frame,
 			HinawaFwTcode lock_tcode, GError **exception)
 {
-	static const HinawaFwTcode lock_tcodes[] = {
-		HINAWA_FW_TCODE_LOCK_RESPONSE,
-		HINAWA_FW_TCODE_LOCK_MASK_SWAP,
-		HINAWA_FW_TCODE_LOCK_COMPARE_SWAP,
-		HINAWA_FW_TCODE_LOCK_FETCH_ADD,
-		HINAWA_FW_TCODE_LOCK_LITTLE_ADD,
-		HINAWA_FW_TCODE_LOCK_BOUNDED_ADD,
-		HINAWA_FW_TCODE_LOCK_WRAP_ADD,
-		HINAWA_FW_TCODE_LOCK_VENDOR_DEPENDENT,
-	};
-	int i;
+	HinawaFwNode *node;
+	guint length = (*frame)->len / 2;
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
-
-	if (!(*frame) || (*frame)->len == 0 || (*frame)->len % 8 > 0 ||
-	    (addr & 0x03)) {
-		raise(exception, EINVAL);
-		return;
-	}
-
-	for (i = 0; i < sizeof(lock_tcodes) / sizeof(lock_tcodes[0]); ++i) {
-		if (lock_tcodes[i] == lock_tcode)
-			break;
-	}
-	if (i == sizeof(lock_tcodes) / sizeof(lock_tcodes[0])) {
-		raise(exception, EINVAL);
-		return;
-	}
-
-	g_object_ref(unit);
-	fw_req_transact(self, unit, lock_tcode, addr, (*frame)->data,
-			(*frame)->len, exception);
-	g_object_unref(unit);
+	hinawa_fw_unit_get_node(unit, &node);
+	g_object_ref(node);
+	hinawa_fw_req_transaction(self, node, lock_tcode, addr, length,
+				&((*frame)->data), &((*frame)->len), exception);
+	g_object_unref(node);
 }
 
 /* NOTE: For HinawaFwUnit, internal. */
