@@ -10,7 +10,7 @@
 /**
  * SECTION:fw_resp
  * @Title: HinawaFwResp
- * @Short_description: A transaction responder for a FireWire unit
+ * @Short_description: A transaction responder for a FireWire node.
  *
  * A HinawaFwResp responds requests from any units.
  *
@@ -25,7 +25,7 @@ G_DEFINE_QUARK("HinawaFwResp", hinawa_fw_resp)
 		    "%d: %s", __LINE__, strerror(errno))
 
 struct _HinawaFwRespPrivate {
-	HinawaFwUnit *unit;
+	HinawaFwNode *node;
 
 	guint width;
 	guint64 addr_handle;
@@ -34,6 +34,9 @@ struct _HinawaFwRespPrivate {
 	unsigned int req_length;
 	guint8 *resp_frame;
 	unsigned int resp_length;
+
+	// For backward compatibility.
+	gboolean registered;
 };
 G_DEFINE_TYPE_WITH_PRIVATE(HinawaFwResp, hinawa_fw_resp, G_TYPE_OBJECT)
 
@@ -48,7 +51,7 @@ static void fw_resp_finalize(GObject *obj)
 {
 	HinawaFwResp *self = HINAWA_FW_RESP(obj);
 
-	hinawa_fw_resp_unregister(self);
+	hinawa_fw_resp_release(self);
 
 	G_OBJECT_CLASS(hinawa_fw_resp_parent_class)->finalize(obj);
 }
@@ -66,7 +69,7 @@ static void hinawa_fw_resp_class_init(HinawaFwRespClass *klass)
 	 * @self: A #HinawaFwResp
 	 * @tcode: One of #HinawaTcode enumerators
 	 *
-	 * When any units transfer requests to the range of address to which
+	 * When any node transfers requests to the range of address to which
 	 * this object listening. The ::requested signal handler can get data
 	 * frame by a call of ::get_req_frame and set data frame by a call of
 	 * ::set_resp_frame, then returns rcode.
@@ -103,17 +106,20 @@ HinawaFwResp *hinawa_fw_resp_new(void)
 }
 
 /**
- * hinawa_fw_resp_register:
- * @self: A #HinawaFwResp
- * @unit: A #HinawaFwUnit
- * @addr: A start address to listen to in host controller
- * @width: The byte width of address to listen to host controller
- * @exception: A #GError
+ * hinawa_fw_resp_reserve:
+ * @self: A #HinawaFwResp.
+ * @node: A #HinawaFwNode.
+ * @addr: A start address to listen to in host controller.
+ * @width: The byte width of address to listen to host controller.
+ * @exception: A #GError.
  *
- * Start to listen to a range of address in host controller
+ * Start to listen to a range of address in host controller which connects to
+ * the node.
+ *
+ * Since: 1.4.
  */
-void hinawa_fw_resp_register(HinawaFwResp *self, HinawaFwUnit *unit,
-			     guint64 addr, guint width, GError **exception)
+void hinawa_fw_resp_reserve(HinawaFwResp *self, HinawaFwNode*node,
+			    guint64 addr, guint width, GError **exception)
 {
 	HinawaFwRespPrivate *priv;
 	struct fw_cdev_allocate allocate = {0};
@@ -122,36 +128,34 @@ void hinawa_fw_resp_register(HinawaFwResp *self, HinawaFwUnit *unit,
 	g_return_if_fail(HINAWA_IS_FW_RESP(self));
 	priv = hinawa_fw_resp_get_instance_private(self);
 
-	if (priv->unit != NULL) {
+	if (priv->node != NULL) {
 		raise(exception, EINVAL);
 		return;
 	}
-	priv->unit = g_object_ref(unit);
 
 	allocate.offset = addr;
 	allocate.closure = (guint64)self;
 	allocate.length = width;
 	allocate.region_end = addr + width;
 
-	hinawa_fw_unit_ioctl(priv->unit, FW_CDEV_IOC_ALLOCATE, &allocate, &err);
+	hinawa_fw_node_ioctl(node, FW_CDEV_IOC_ALLOCATE, &allocate, &err);
 	if (err != 0) {
 		raise(exception, err);
-		g_object_unref(priv->unit);
-		priv->unit = NULL;
 		return;
 	}
+	priv->node = g_object_ref(node);
 
 	priv->req_frame = g_malloc(allocate.length);
 	if (!priv->req_frame) {
 		raise(exception, ENOMEM);
-		hinawa_fw_resp_unregister(self);
+		hinawa_fw_resp_release(self);
 		return;
 	}
 
 	priv->resp_frame = g_malloc0(allocate.length);
 	if (!priv->resp_frame) {
 		raise(exception, ENOMEM);
-		hinawa_fw_resp_unregister(self);
+		hinawa_fw_resp_release(self);
 		return;
 	}
 
@@ -160,12 +164,14 @@ void hinawa_fw_resp_register(HinawaFwResp *self, HinawaFwUnit *unit,
 }
 
 /**
- * hinawa_fw_resp_unregister:
- * @self: A HinawaFwResp
+ * hinawa_fw_resp_release:
+ * @self: A HinawaFwResp.
  *
- * stop to listen to a range of address in host controller
+ * stop to listen to a range of address in host controller.
+ *
+ * Since: 1.4.
  */
-void hinawa_fw_resp_unregister(HinawaFwResp *self)
+void hinawa_fw_resp_release(HinawaFwResp *self)
 {
 	HinawaFwRespPrivate *priv;
 	struct fw_cdev_deallocate deallocate = {0};
@@ -174,14 +180,14 @@ void hinawa_fw_resp_unregister(HinawaFwResp *self)
 	g_return_if_fail(HINAWA_IS_FW_RESP(self));
 	priv = hinawa_fw_resp_get_instance_private(self);
 
-	if (priv->unit == NULL)
+	if (priv->node == NULL)
 		return;
 
 	deallocate.handle = priv->addr_handle;
-	hinawa_fw_unit_ioctl(priv->unit, FW_CDEV_IOC_DEALLOCATE, &deallocate,
+	hinawa_fw_node_ioctl(priv->node, FW_CDEV_IOC_DEALLOCATE, &deallocate,
 			     &err);
-	g_object_unref(priv->unit);
-	priv->unit = NULL;
+	g_object_unref(priv->node);
+	priv->node = NULL;
 
 	if (priv->req_frame != NULL)
 		g_free(priv->req_frame);
@@ -193,11 +199,64 @@ void hinawa_fw_resp_unregister(HinawaFwResp *self)
 	priv->resp_length = 0;
 }
 
+/**
+ * hinawa_fw_resp_register:
+ * @self: A #HinawaFwResp
+ * @unit: A #HinawaFwUnit
+ * @addr: A start address to listen to in host controller
+ * @width: The byte width of address to listen to host controller
+ * @exception: A #GError
+ *
+ * Start to listen to a range of address in host controller
+ *
+ * Deprecated: 1.4: Use hinawa_fw_resp_reserve() with an instance of
+ *		    HinawaFwNode, instead.
+ */
+void hinawa_fw_resp_register(HinawaFwResp *self, HinawaFwUnit *unit,
+			     guint64 addr, guint width, GError **exception)
+{
+	HinawaFwRespPrivate *priv;
+	HinawaFwNode *node;
+
+	g_return_if_fail(HINAWA_IS_FW_RESP(self));
+	priv = hinawa_fw_resp_get_instance_private(self);
+
+	hinawa_fw_unit_get_node(unit, &node);
+	g_object_ref(node);
+	hinawa_fw_resp_reserve(self, node, addr, width, exception);
+	g_object_unref(node);
+
+	if (*exception != NULL)
+		return;
+
+	priv->registered = TRUE;
+}
+
+/**
+ * hinawa_fw_resp_unregister:
+ * @self: A HinawaFwResp
+ *
+ * stop to listen to a range of address in host controller
+ *
+ * Deprecated: 1.4: Use hinawa_fw_resp_release() with an instance of
+ *		    HinawaFwNode, instead.
+ */
+void hinawa_fw_resp_unregister(HinawaFwResp *self)
+{
+	HinawaFwRespPrivate *priv;
+
+	g_return_if_fail(HINAWA_IS_FW_RESP(self));
+	priv = hinawa_fw_resp_get_instance_private(self);
+
+	hinawa_fw_resp_release(self);
+	priv->registered = FALSE;
+}
+
 /**
  * hinawa_fw_resp_get_req_frame:
  * @self: A #HinawaFwResp
- * @frame: (element-type guint8)(array length=length)(out): a 8bit array for
- * 	   response frame.
+ * @frame: (array length=length)(out)(transfer none): a 8bit array for response
+ * 	   frame.
  * @length: (out): The length of bytes for the frame.
  *
  * Retrieve byte frame to be requested.
@@ -263,7 +322,7 @@ static void fw_resp_notify_requested(void *target, void *data,
 	}
 
 	resp.handle = event->handle;
-	hinawa_fw_unit_ioctl(priv->unit, FW_CDEV_IOC_SEND_RESPONSE, &resp,
+	hinawa_fw_node_ioctl(priv->node, FW_CDEV_IOC_SEND_RESPONSE, &resp,
 			     &err);
 
 	memset(priv->resp_frame, 0, priv->width);
@@ -282,20 +341,23 @@ void hinawa_fw_resp_handle_request(HinawaFwResp *self,
 	g_return_if_fail(HINAWA_IS_FW_RESP(self));
 	priv = hinawa_fw_resp_get_instance_private(self);
 
-	if (!priv->unit || event->length > priv->width) {
+	if (!priv->node || event->length > priv->width) {
 		resp.rcode = RCODE_CONFLICT_ERROR;
-		goto error;
+		resp.handle = event->handle;
+
+		hinawa_fw_node_ioctl(priv->node, FW_CDEV_IOC_SEND_RESPONSE,
+				     &resp, &err);
+		return;
 	}
 
 	length = sizeof(*event) + event->length;
 
-	// Emit signal to handlers later.
-	hinawa_context_schedule_notification(self, event, length,
+	// For backward compatibility.
+	if (priv->registered) {
+		hinawa_context_schedule_notification(self, event, length,
 					     fw_resp_notify_requested, &err);
-	return;
-error:
-	resp.handle = event->handle;
+		return;
+	}
 
-	hinawa_fw_unit_ioctl(priv->unit, FW_CDEV_IOC_SEND_RESPONSE,
-			     &resp, &err);
+	fw_resp_notify_requested(self, (void *)event, length);
 }
