@@ -64,6 +64,10 @@ GType hinawa_fw_resp_get_type(void) G_GNUC_CONST;
 
 HinawaFwResp *hinawa_fw_resp_new(void);
 
+void hinawa_fw_resp_reserve(HinawaFwResp *self, HinawaFwNode*node,
+			    guint64 addr, guint width, GError **exception);
+void hinawa_fw_resp_release(HinawaFwResp *self);
+
 void hinawa_fw_resp_register(HinawaFwResp *self, HinawaFwUnit *unit,
 			     guint64 addr, guint width, GError **exception);
 void hinawa_fw_resp_unregister(HinawaFwResp *self);
