@@ -9,7 +9,7 @@
 
 #include "internal.h"
 
-/* For error handling. */
+// For error handling.
 G_DEFINE_QUARK("HinawaFwUnit", hinawa_fw_unit)
 #define raise(exception, errno)						\
 	g_set_error(exception, hinawa_fw_unit_quark(), errno,		\
@@ -18,31 +18,26 @@ G_DEFINE_QUARK("HinawaFwUnit", hinawa_fw_unit)
 /**
  * SECTION:fw_unit
  * @Title: HinawaFwUnit
- * @Short_description: An event listener for FireWire unit
+ * @Short_description: An object to maintain association between IEEE 1394 unit
+ *		       and its parent.
  *
- * A #HinawaFwUnit is an event listener for a certain FireWire unit.
- * This class is an application of Linux FireWire subsystem.
- * All of operations utilize ioctl(2) with subsystem specific request commands.
+ * A #HinawaFwUnit object has a reference to an instance of HinawaFwNode for
+ * Linux FireWire character device, corresponding to parent node on IEEE 1394
+ * bus. This object is expected to be used by inheritance from subclasses; e.g.
+ * HinawaSndUnit, and should not be instantiated directly for newly written
+ * code since the object is planned to be abstract class in future libhinawa
+ * release.
+ *
+ * All of operations are done by associated HinawaFwNode. Some APIs and
+ * properties which #HinawaFwUnit has are maintained just for backward
+ * compatibility and already deprecated. Instead, use associated HinawaFwNode.
  */
 
-/*
- * 256 comes from an actual implementation in kernel land. Read
- * 'drivers/firewire/core-device.c'. This value is calculated by a range for
- * configuration ROM in ISO/IEC 13213 (IEEE 1212).
- */
-#define MAX_CONFIG_ROM_SIZE	256
-#define MAX_CONFIG_ROM_LENGTH	(MAX_CONFIG_ROM_SIZE * 4)
-
 struct _HinawaFwUnitPrivate {
-	int fd;
-
-	GMutex mutex;
-	guint8 config_rom[MAX_CONFIG_ROM_LENGTH];
-	unsigned int config_rom_length;
-	struct fw_cdev_event_bus_reset generation;
-
+	HinawaFwNode *node;
 	GSource *src;
 };
+// TODO: use G_DEFINE_ABSTRACT_TYPE().
 G_DEFINE_TYPE_WITH_PRIVATE(HinawaFwUnit, hinawa_fw_unit, G_TYPE_OBJECT)
 
 typedef struct {
@@ -53,7 +48,7 @@ typedef struct {
 	void *buf;
 } FwUnitSource;
 
-/* This object has properties. */
+// This object has deprecated properties.
 enum fw_unit_prop_type {
 	FW_UNIT_PROP_TYPE_NODE_ID = 1,
 	FW_UNIT_PROP_TYPE_LOCAL_NODE_ID,
@@ -66,7 +61,7 @@ enum fw_unit_prop_type {
 };
 static GParamSpec *fw_unit_props[FW_UNIT_PROP_TYPE_COUNT] = { NULL, };
 
-/* This object has one signal. */
+// This object has two deprecated signals.
 enum fw_unit_sig_type {
 	FW_UNIT_SIG_TYPE_BUS_UPDATE = 0,
 	FW_UNIT_SIG_TYPE_DISCONNECTED,
@@ -79,37 +74,16 @@ static void fw_unit_get_property(GObject *obj, guint id,
 {
 	HinawaFwUnit *self = HINAWA_FW_UNIT(obj);
 	HinawaFwUnitPrivate *priv = hinawa_fw_unit_get_instance_private(self);
+	GObject *node = G_OBJECT(priv->node);
 
 	switch (id) {
 	case FW_UNIT_PROP_TYPE_NODE_ID:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.node_id);
-		g_mutex_unlock(&priv->mutex);
-		break;
 	case FW_UNIT_PROP_TYPE_LOCAL_NODE_ID:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.local_node_id);
-		g_mutex_unlock(&priv->mutex);
-		break;
 	case FW_UNIT_PROP_TYPE_BUS_MANAGER_NODE_ID:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.bm_node_id);
-		g_mutex_unlock(&priv->mutex);
-		break;
 	case FW_UNIT_PROP_TYPE_IR_MANAGER_NODE_ID:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.irm_node_id);
-		g_mutex_unlock(&priv->mutex);
-		break;
 	case FW_UNIT_PROP_TYPE_ROOT_NODE_ID:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.root_node_id);
-		g_mutex_unlock(&priv->mutex);
-		break;
 	case FW_UNIT_PROP_TYPE_GENERATION:
-		g_mutex_lock(&priv->mutex);
-		g_value_set_ulong(val, priv->generation.generation);
-		g_mutex_unlock(&priv->mutex);
+		g_object_get_property(node, fw_unit_props[id]->name, val);
 		break;
 	case FW_UNIT_PROP_TYPE_LISTENING:
 		g_value_set_boolean(val, priv->src != NULL);
@@ -133,9 +107,7 @@ static void fw_unit_finalize(GObject *obj)
 
 	hinawa_fw_unit_unlisten(self);
 
-	close(priv->fd);
-
-	g_mutex_clear(&priv->mutex);
+	g_object_unref(priv->node);
 
 	G_OBJECT_CLASS(hinawa_fw_unit_parent_class)->finalize(obj);
 }
@@ -152,42 +124,42 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 		g_param_spec_ulong("node-id", "node-id",
 				   "Node-ID of this unit at this generation.",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_LOCAL_NODE_ID] =
 		g_param_spec_ulong("local-node-id", "local-node-id",
 				   "Node-ID for a unit which this unit use to "
 				   "communicate to the other units on the bus "
 				   "at this generation.",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_BUS_MANAGER_NODE_ID] =
 		g_param_spec_ulong("bus-manager-node-id", "bus-manager-node-id",
 				   "Node-ID for bus manager on the bus at this "
 				   "generation.",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_IR_MANAGER_NODE_ID] =
 		g_param_spec_ulong("ir-manager-node-id", "ir-manager-node-id",
 				   "Node-ID for isochronous resource manager "
 				   "on the bus at this generation",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_ROOT_NODE_ID] =
 		g_param_spec_ulong("root-node-id", "root-node-id",
 				   "Node-ID for root of bus topology at this "
 				   "generation.",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_GENERATION] =
 		g_param_spec_ulong("generation", "generation",
 				   "current level of generation on this bus.",
 				   0, ULONG_MAX, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READABLE | G_PARAM_DEPRECATED);
 	fw_unit_props[FW_UNIT_PROP_TYPE_LISTENING] =
 		g_param_spec_boolean("listening", "listening",
 				     "Whether this device is under listening.",
 				     FALSE,
-				     G_PARAM_READABLE);
+				     G_PARAM_READABLE | G_PARAM_DEPRECATED);
 
 	g_object_class_install_properties(gobject_class,
 					  FW_UNIT_PROP_TYPE_COUNT,
@@ -200,6 +172,9 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 	 * When IEEE 1394 bus is updated, the ::bus-update signal is generated.
 	 * Handlers can read current generation in the bus via 'generation'
 	 * property.
+	 *
+	 * Deprecated: 1.4: Use an instance of HinawaFwNode retrieved by a call
+	 *		    of hinawa_fw_unit_get_node().
 	 */
 	fw_unit_sigs[FW_UNIT_SIG_TYPE_BUS_UPDATE] =
 		g_signal_new("bus-update",
@@ -214,8 +189,11 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 	 * HinawaFwUnit::disconnected:
 	 * @self: A #HinawaFwUnit
 	 *
-	 * When phicical FireWire devices are disconnected from IEEE 1394 bus,
-	 * the #HinawaFwUnit becomes unlostening and emits this signal.
+	 * When physical FireWire devices are disconnected from IEEE 1394 bus,
+	 * the #HinawaFwUnit becomes unlistening and emits this signal.
+	 *
+	 * Deprecated: 1.4: Use an instance of HinawaFwNode retrieved by a call
+	 *		    of hinawa_fw_unit_get_node().
 	 */
 	fw_unit_sigs[FW_UNIT_SIG_TYPE_DISCONNECTED] =
 		g_signal_new("disconnected",
@@ -230,7 +208,7 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 static void hinawa_fw_unit_init(HinawaFwUnit *self)
 {
 	HinawaFwUnitPrivate *priv= hinawa_fw_unit_get_instance_private(self);
-	g_mutex_init(&priv->mutex);
+	priv->node = hinawa_fw_node_new();
 }
 
 /**
@@ -240,105 +218,14 @@ static void hinawa_fw_unit_init(HinawaFwUnit *self)
  *
  * Returns: an instance of #HinawaFwUnit.
  * Since: 1.3.
+ * Deprecated: 1.4: HinawaFwUnit is planned to be an abstract class in future
+ *		    release. Please instantiate for derived class, instead.
  */
 HinawaFwUnit *hinawa_fw_unit_new(void)
 {
 	return g_object_new(HINAWA_TYPE_FW_UNIT, NULL);
 }
 
-static void update_info(HinawaFwUnit *self, GError **exception)
-{
-	struct fw_cdev_get_info info = {0};
-	HinawaFwUnitPrivate *priv = hinawa_fw_unit_get_instance_private(self);
-	guint32 *rom;
-	unsigned int quads;
-	int i;
-
-	/* Duplicate generation parameters in userspace. */
-	info.version = 4;
-	info.rom = (__u64)priv->config_rom;
-	info.rom_length = MAX_CONFIG_ROM_LENGTH;
-	info.bus_reset = (guint64)&priv->generation;
-	info.bus_reset_closure = (guint64)self;
-
-	if (ioctl(priv->fd, FW_CDEV_IOC_GET_INFO, &info) < 0)
-		raise(exception, errno);
-
-	/*
-	 * Align buffer for configuration ROM according to host endianness, because
-	 * Linux firewire subsystem copies raw data to it.
-	 */
-	rom = (guint32 *)priv->config_rom;
-	quads = (info.rom_length + 3) / 4;
-	for (i = 0; i < quads; ++i)
-		rom[i] = GUINT32_FROM_BE(rom[i]);
-	priv->config_rom_length = info.rom_length;
-}
-
-/**
- * hinawa_fw_unit_open:
- * @self: A #HinawaFwUnit
- * @path: A path to Linux FireWire character device
- * @exception: A #GError
- *
- * Open Linux FireWire character device to operate for node on IEEE 1394 bus.
- */
-void hinawa_fw_unit_open(HinawaFwUnit *self, gchar *path, GError **exception)
-{
-	HinawaFwUnitPrivate *priv;
-	int fd;
-
-	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
-	priv = hinawa_fw_unit_get_instance_private(self);
-
-	fd = open(path, O_RDONLY);
-	if (fd < 0) {
-		raise(exception, errno);
-		return;
-	}
-	priv->fd = fd;
-
-	g_mutex_lock(&priv->mutex);
-	update_info(self, exception);
-	g_mutex_unlock(&priv->mutex);
-}
-
-/**
- * hinawa_fw_unit_get_config_rom:
- * @self: A #HinawaFwUnit
- * @length: (out) (optional): the number of bytes consists of the config rom
- *
- * Get cached content of configuration ROM.
- *
- * Returns: (element-type guint8) (array length=length) (transfer none): config rom image
- */
-const guint8 *hinawa_fw_unit_get_config_rom(HinawaFwUnit *self, guint *length)
-{
-	HinawaFwUnitPrivate *priv;
-
-	g_return_val_if_fail(HINAWA_IS_FW_UNIT(self), NULL);
-	priv = hinawa_fw_unit_get_instance_private(self);
-
-	if (length)
-		*length = priv->config_rom_length;
-
-	return priv->config_rom;
-}
-
-/* Internal use only. */
-void hinawa_fw_unit_ioctl(HinawaFwUnit *self, unsigned long req, void *args,
-			  int *err)
-{
-	HinawaFwUnitPrivate *priv;
-
-	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
-	priv = hinawa_fw_unit_get_instance_private(self);
-
-	*err = 0;
-	if (ioctl(priv->fd, req, args) < 0)
-		*err = errno;
-}
-
 static void fw_unit_notify_update(void *target, void *data, unsigned int length)
 {
 	HinawaFwUnit *self = target;
@@ -354,131 +241,92 @@ static void fw_unit_notify_disconnected(void *target, void *data,
 	g_signal_emit(self, fw_unit_sigs[FW_UNIT_SIG_TYPE_DISCONNECTED], 0);
 }
 
-static void handle_update(HinawaFwUnit *self,
-			  struct fw_cdev_event_bus_reset *event)
+static void handle_bus_update(HinawaFwNode *node, gpointer arg)
 {
-	HinawaFwUnitPrivate *priv;
-	int err = 0;
+	HinawaFwUnit *self = (HinawaFwUnit *)arg;
+	HinawaFwUnitPrivate *priv = hinawa_fw_unit_get_instance_private(self);
+	int err;
 
-	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
-	priv = hinawa_fw_unit_get_instance_private(self);
-
-	g_mutex_lock(&priv->mutex);
-	update_info(self, NULL);
-	g_mutex_unlock(&priv->mutex);
-
-	hinawa_context_schedule_notification(self, NULL, 0,
+	// For backward compatibility.
+	if (priv->src != NULL) {
+		hinawa_context_schedule_notification(self, NULL, 0,
 					     fw_unit_notify_update, &err);
-}
-
-static gboolean prepare_src(GSource *src, gint *timeout)
-{
-	// Use blocking poll(2) to save CPU usage.
-	*timeout = -1;
-
-	/* This source is not ready, let's poll(2) */
-	return FALSE;
-}
-
-static gboolean check_src(GSource *gsrc)
-{
-	FwUnitSource *src = (FwUnitSource *)gsrc;
-	GIOCondition condition;
-
-	condition = g_source_query_unix_fd(gsrc, src->tag);
-	if (condition & G_IO_ERR) {
-		HinawaFwUnit *unit = src->unit;
-		int err = 0;
-
-		if (unit != NULL) {
-			hinawa_fw_unit_unlisten(unit);
-
-			hinawa_context_schedule_notification(unit, NULL, 0,
-						fw_unit_notify_disconnected,
-						&err);
-		}
-	}
-
-	// Don't go to dispatch if nothing available.
-	return !!(condition & G_IO_IN);
-}
-
-static gboolean dispatch_src(GSource *gsrc, GSourceFunc cb, gpointer user_data)
-{
-	FwUnitSource *src = (FwUnitSource *)gsrc;
-	HinawaFwUnit *unit = src->unit;
-	HinawaFwUnitPrivate *priv;
-	struct fw_cdev_event_common *common;
-	int len;
-
-	if (unit == NULL)
-		goto end;
-	priv = hinawa_fw_unit_get_instance_private(unit);
-
-	len = read(priv->fd, src->buf, src->len);
-	if (len <= 0)
-		goto end;
-
-	common = (struct fw_cdev_event_common *)src->buf;
-
-	if (HINAWA_IS_FW_UNIT(common->closure) &&
-	    common->type == FW_CDEV_EVENT_BUS_RESET)
-		handle_update(HINAWA_FW_UNIT(common->closure),
-				(struct fw_cdev_event_bus_reset *)common);
-	else if (HINAWA_IS_FW_RESP(common->closure) &&
-		 common->type == FW_CDEV_EVENT_REQUEST2)
-		hinawa_fw_resp_handle_request(HINAWA_FW_RESP(common->closure),
-				(struct fw_cdev_event_request2 *)common);
-	else if (HINAWA_IS_FW_REQ(common->closure) &&
-		 common->type == FW_CDEV_EVENT_RESPONSE)
-		hinawa_fw_req_handle_response(HINAWA_FW_REQ(common->closure),
-				(struct fw_cdev_event_response *)common);
-end:
-	/* Just be sure to continue to process this source. */
-	return G_SOURCE_CONTINUE;
-}
-
-static void finalize_src(GSource *gsrc)
-{
-	FwUnitSource *src = (FwUnitSource *)gsrc;
-
-	g_free(src->buf);
-}
-
-static void fw_unit_create_source(HinawaFwUnit *self, GSource **gsrc,
-				  GError **exception)
-{
-	static GSourceFuncs funcs = {
-		.prepare	= prepare_src,
-		.check		= check_src,
-		.dispatch	= dispatch_src,
-		.finalize	= finalize_src,
-	};
-	HinawaFwUnitPrivate *priv;
-	FwUnitSource *src;
-
-	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
-	priv = hinawa_fw_unit_get_instance_private(self);
-
-	*gsrc = g_source_new(&funcs, sizeof(FwUnitSource));
-
-	g_source_set_name(*gsrc, "HinawaFwUnit");
-	g_source_set_priority(*gsrc, G_PRIORITY_HIGH_IDLE);
-	g_source_set_can_recurse(*gsrc, TRUE);
-
-	// MEMO: allocate one page because we cannot assume the size of
-	// transaction frame.
-	src = (FwUnitSource *)(*gsrc);
-	src->len = sysconf(_SC_PAGESIZE);
-	src->buf = g_malloc0(src->len);
-	if (src->buf == NULL) {
-		raise(exception, ENOMEM);
-		g_source_unref(*gsrc);
 		return;
 	}
 
-	src->unit = self;
-	src->tag = g_source_add_unix_fd(*gsrc, priv->fd, G_IO_IN);
+	fw_unit_notify_update(self, NULL, 0);
+}
+
+static void handle_disconnected(HinawaFwNode *node, gpointer arg)
+{
+	HinawaFwUnit *self = (HinawaFwUnit *)arg;
+	HinawaFwUnitPrivate *priv = hinawa_fw_unit_get_instance_private(self);
+	int err;
+
+	// For backward compatibility.
+	if (priv->src != NULL) {
+		hinawa_context_schedule_notification(self, NULL, 0,
+					     fw_unit_notify_disconnected, &err);
+		return;
+	}
+
+	fw_unit_notify_disconnected(self, NULL, 0);
+}
+
+/**
+ * hinawa_fw_unit_open:
+ * @self: A #HinawaFwUnit
+ * @path: A path to Linux FireWire character device
+ * @exception: A #GError
+ *
+ * Open Linux FireWire character device to operate for node on IEEE 1394 bus.
+ */
+void hinawa_fw_unit_open(HinawaFwUnit *self, gchar *path, GError **exception)
+{
+	HinawaFwUnitPrivate *priv;
+
+	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
+	priv = hinawa_fw_unit_get_instance_private(self);
+
+	hinawa_fw_node_open(priv->node, path, exception);
+	if (*exception != NULL)
+		return;
+
+	g_signal_connect(G_OBJECT(priv->node), "bus_update",
+			 G_CALLBACK(handle_bus_update), self);
+	g_signal_connect(G_OBJECT(priv->node), "disconnected",
+			 G_CALLBACK(handle_disconnected), self);
+}
+
+/**
+ * hinawa_fw_unit_get_config_rom:
+ * @self: A #HinawaFwUnit
+ * @length: (out)(optional): the number of bytes consists of the config rom
+ *
+ * Get cached content of configuration ROM.
+ *
+ * Returns: (element-type guint8)(array length=length)(transfer none): config
+ *	    rom image.
+ *
+ * Deprecated: 1.4: Instead, use hinawa_fw_node_get_config_rom() for an instance
+ *		    of HinawaFwNode retrieved by a call of
+ *		    hinawa_fw_unit_get_node().
+ */
+const guint8 *hinawa_fw_unit_get_config_rom(HinawaFwUnit *self, guint *length)
+{
+	HinawaFwUnitPrivate *priv;
+	const guint8 *image;
+	GError *exception;
+
+	g_return_val_if_fail(HINAWA_IS_FW_UNIT(self), NULL);
+	priv = hinawa_fw_unit_get_instance_private(self);
+
+	exception = NULL;
+	hinawa_fw_node_get_config_rom(priv->node, &image, length, &exception);
+	if (exception != NULL)
+		return NULL;
+
+	return image;
 }
 
 /**
@@ -487,6 +335,12 @@ static void fw_unit_create_source(HinawaFwUnit *self, GSource **gsrc,
  * @exception: A #GError
  *
  * Start to listen to any events from the unit.
+ *
+ * Deprecated: 1.4: Instead, use GSource retrieved by a call of
+ *		    hinawa_fw_node_create_source() for an instance of
+ *		    HinawaFwNode retrieved by a call of
+ *		    hinawa_fw_unit_get_node(). Then use GMainContext and
+ *		    GMainLoop of GLib for event loop.
  */
 void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 {
@@ -495,15 +349,24 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
 	priv = hinawa_fw_unit_get_instance_private(self);
 
-	fw_unit_create_source(self, &priv->src, exception);
-	if (*exception != NULL)
-		return;
+	if (priv->src == NULL) {
+		hinawa_fw_node_create_source(priv->node, &priv->src, exception);
+		if (*exception != NULL) {
+			hinawa_fw_unit_unlisten(self);
+			return;
+		}
 
-	hinawa_context_add_src(priv->src, exception);
-	if (*exception != NULL) {
-		g_source_unref(priv->src);
-		priv->src = NULL;
-		return;
+		hinawa_context_add_src(priv->src, exception);
+		if (*exception != NULL) {
+			hinawa_fw_unit_unlisten(self);
+			return;
+		}
+
+		hinawa_context_start_notifier(exception);
+		if (*exception != NULL) {
+			hinawa_fw_unit_unlisten(self);
+			return;
+		}
 	}
 }
 
@@ -512,6 +375,9 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
  * @self: A #HinawaFwUnit
  *
  * Stop to listen to any events from the unit.
+ *
+ * Deprecated: 1.4: Instead, maintain GMainContext and GMainLoop with GSource
+ *		    retrieved by a call of hinawa_fw_node_create_source().
  */
 void hinawa_fw_unit_unlisten(HinawaFwUnit *self)
 {
@@ -521,8 +387,29 @@ void hinawa_fw_unit_unlisten(HinawaFwUnit *self)
 	priv = hinawa_fw_unit_get_instance_private(self);
 
 	if (priv->src != NULL) {
+		hinawa_context_stop_notifier();
+
 		hinawa_context_remove_src(priv->src);
 		g_source_unref(priv->src);
 		priv->src = NULL;
 	}
 }
+
+/**
+ * hinawa_fw_unit_get_node:
+ * @self: A #HinawaFwUnit.
+ * @node: (out)(transfer none): A #HinawaFwNode.
+ *
+ * Retrieve an instance of #HinawaFwNode associated to the given unit.
+ *
+ * Since: 1.4.
+ */
+void hinawa_fw_unit_get_node(HinawaFwUnit *self, HinawaFwNode **node)
+{
+	HinawaFwUnitPrivate *priv;
+
+	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
+	priv = hinawa_fw_unit_get_instance_private(self);
+
+	*node = priv->node;
+}
