@@ -13,6 +13,7 @@
 #include <sound/firewire.h>
 #endif
 
+#include "fw_node.h"
 #include "fw_unit.h"
 #include "fw_resp.h"
 #include "fw_req.h"
@@ -26,13 +27,17 @@
 void hinawa_context_add_src(GSource *src, GError **exception);
 void hinawa_context_remove_src(GSource *src);
 
+void hinawa_context_start_notifier(GError **exception);
+void hinawa_context_stop_notifier();
+
 typedef void (*NotifierWorkFunc)(void *target, void *data, unsigned int length);
 void hinawa_context_schedule_notification(void *target, const void *data,
 				unsigned int length, NotifierWorkFunc func,
 				int *err);
 
-void hinawa_fw_unit_ioctl(HinawaFwUnit *self, unsigned long req, void *args,
+void hinawa_fw_node_ioctl(HinawaFwNode *self, unsigned long req, void *args,
 			  int *err);
+
 void hinawa_fw_resp_handle_request(HinawaFwResp *self,
 				   struct fw_cdev_event_request2 *event);
 void hinawa_fw_req_handle_response(HinawaFwReq *self,
