@@ -153,7 +153,7 @@ static void hinawa_snd_unit_class_init(HinawaSndUnitClass *klass)
 		g_param_spec_boolean("listening", "listening",
 				     "Whether this device is under listening.",
 				     FALSE,
-				     G_PARAM_READABLE);
+				     G_PARAM_READABLE | G_PARAM_DEPRECATED);
 
 	g_object_class_install_properties(gobject_class,
 					  SND_UNIT_PROP_TYPE_COUNT,
@@ -225,8 +225,6 @@ void hinawa_snd_unit_open(HinawaSndUnit *self, gchar *path, GError **exception)
 
 	snprintf(fw_cdev, sizeof(fw_cdev), "/dev/%s", priv->info.device_name);
 	hinawa_fw_unit_open(&self->parent_instance, fw_cdev, exception);
-	if (*exception != NULL)
-		goto end;
 end:
 	if (*exception != NULL)
 		close(priv->fd);
@@ -297,8 +295,11 @@ void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
 static void snd_unit_notify_lock(void *target, void *data, unsigned int length)
 {
 	HinawaSndUnit *self = target;
+	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(self);
 	struct snd_firewire_event_lock_status *event = data;
 
+	priv->streaming = event->status;
+
 	g_signal_emit(self, snd_unit_sigs[SND_UNIT_SIG_TYPE_LOCK_STATUS], 0,
 		      event->status);
 }
@@ -314,10 +315,17 @@ static void snd_unit_notify_disconnected(void *target, void *data,
 static void handle_lock_event(HinawaSndUnit *self,
 			      void *buf, unsigned int length)
 {
+	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(self);
 	int err = 0;
 
-	hinawa_context_schedule_notification(self, buf, length,
+	// For backward compatibility.
+	if (priv->src != NULL) {
+		hinawa_context_schedule_notification(self, buf, length,
 					     snd_unit_notify_lock, &err);
+		return;
+	}
+
+	snd_unit_notify_lock(self, (void *)buf, length);
 }
 
 static gboolean prepare_src(GSource *src, gint *timeout)
@@ -332,26 +340,25 @@ static gboolean prepare_src(GSource *src, gint *timeout)
 static gboolean check_src(GSource *gsrc)
 {
 	SndUnitSource *src = (SndUnitSource *)gsrc;
+	HinawaSndUnit *unit = src->unit;
+	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(unit);
 	GIOCondition condition;
 
 	condition = g_source_query_unix_fd(gsrc, src->tag);
 	if (condition & G_IO_ERR) {
-		HinawaSndUnit *unit = src->unit;
 		if (unit != NULL) {
-			gboolean listening;
 			int err = 0;
 
-			// For emitting one signal.
-			g_object_get(G_OBJECT(&unit->parent_instance),
-				     "listening", &listening, NULL);
-
-			hinawa_snd_unit_unlisten(unit);
-
-			if (listening) {
+			if (priv->src != NULL) {
+				// For backward compatibility.
 				hinawa_context_schedule_notification(unit,
 					NULL, 0, snd_unit_notify_disconnected,
 					&err);
+			} else {
+				snd_unit_notify_disconnected(unit, NULL, 0);
 			}
+
+			hinawa_snd_unit_unlisten(unit);
 		}
 	}
 
@@ -426,7 +433,17 @@ static void finalize_src(GSource *gsrc)
 	g_free(src->buf);
 }
 
-static void snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
+/**
+ * hinawa_snd_unit_create_source:
+ * @self: A #HinawaSndUnit.
+ * @gsrc: (out): A #GSource.
+ * @exception: A #GError.
+ *
+ * Create Gsource for GMainContext to dispatch events for the sound device.
+ *
+ * Since: 1.4.
+ */
+void hinawa_snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
 				   GError **exception)
 {
 	static GSourceFuncs funcs = {
@@ -455,6 +472,7 @@ static void snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
 	if (src->buf == NULL) {
 		raise(exception, ENOMEM);
 		g_source_unref(*gsrc);
+		hinawa_context_stop_notifier();
 		return;
 	}
 
@@ -476,6 +494,10 @@ static void snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
  * @exception: A #GError
  *
  * Start listening to events.
+ *
+ * Deprecated: 1.4: Instead, use GSource retrieved by a call of
+ *		    hinawa_snd_unit_create_source(). Then use GMainContext and
+ *		    GMainLoop of GLib for event loop.
  */
 void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 {
@@ -484,7 +506,7 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
 	priv = hinawa_snd_unit_get_instance_private(self);
 
-	snd_unit_create_source(self, &priv->src, exception);
+	hinawa_snd_unit_create_source(self, &priv->src, exception);
 	if (*exception != NULL)
 		return;
 
@@ -495,6 +517,12 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 		return;
 	}
 
+	hinawa_context_start_notifier(exception);
+	if (*exception != NULL) {
+		hinawa_snd_unit_unlisten(self);
+		return;
+	}
+
 	hinawa_fw_unit_listen(&self->parent_instance, exception);
 	if (*exception != NULL)
 		hinawa_snd_unit_unlisten(self);
@@ -505,6 +533,9 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
  * @self: A #HinawaSndUnit
  *
  * Stop listening to events.
+ *
+ * Deprecated: 1.4: Instead, maintain GMainContext and GMainLoop with GSource
+ *		    retrieved by a call of hinawa_snd_unit_create_source().
  */
 void hinawa_snd_unit_unlisten(HinawaSndUnit *self)
 {
@@ -514,6 +545,8 @@ void hinawa_snd_unit_unlisten(HinawaSndUnit *self)
 	priv = hinawa_snd_unit_get_instance_private(self);
 
 	if (priv->src != NULL) {
+		hinawa_context_stop_notifier();
+
 		hinawa_context_remove_src(priv->src);
 		g_source_unref(priv->src);
 		priv->src = NULL;
